# -*- coding: utf-8 -*-
# !/usr/bin/python

import unittest

import radiusd
from abfab_idp import AbfabIDP, AbfabIDPError

__author__ = "Alejandro Pérez <alex.perez-mendez@jisc.ac.uk>"


class TestAbfabIDP(unittest.TestCase):
    def setUp(self):
        self.conf = {
            "assertion_builder": {
                "type": "pysaml",
                "issuer": "http://myradius.org/",
                "lifetime": 600,
            },
            "attribute_sources": [
                {
                    "type": "static",
                    "static_name": "testattr1",
                    "static_value": "val1"
                }
            ],
            'attribute_filters': [
                {
                    'type': 'combine',
                    'sources': ['urn:oid:2.5.4.3', 'idp_software'],
                    'template': '{0} {1}',
                    'destination': 'super'
                }
            ],
        }

    def test_emtpy_conf(self):
        AbfabIDP({})

    def test_no_username(self):
        idp = AbfabIDP(self.conf)
        code, attributes, _ = idp.post_auth([
            ('RADATTR1', 'whatever'),
            ('RADATTR2', 'whatever'),
            ('RADATTR3', 'whatever'),
            ('RADATTR4', 'whatever'),
        ])
        self.assertEqual(code, radiusd.RLM_MODULE_NOOP)

    def test_valid1(self):
        idp = AbfabIDP(self.conf)
        code, attributes, _ = idp.post_auth([
            ('User-Name', 'alex@test.org'),
            ('RADATTR1', 'whatever'),
            ('RADATTR2', 'whatever'),
            ('RADATTR3', 'whatever'),
            ('RADATTR4', 'whatever'),
        ])
        self.assertEqual(code, radiusd.RLM_MODULE_OK)

    def test_invalid_source(self):
        conf = {
            "assertion_builder": {
                "type": "pysaml",
                "issuer": "http://myradius.org/",
                "lifetime": 600,
            },
            "attribute_sources": [
                {
                    "type": "staticNO",
                    "static_name": "testattr1",
                    "static_value": "val1"
                }
            ]
        }
        with self.assertRaises(AbfabIDPError):
            AbfabIDP(conf)

    def test_failing_source_conf(self):
        conf = {
            "assertion_builder": {
                "type": "pysaml",
                "issuer": "http://myradius.org/",
                "lifetime": 600,
            },
            "attribute_sources": [
                {
                    "type": "ldap",
                }
            ]
        }
        with self.assertRaises(AbfabIDPError):
            AbfabIDP(conf)

    def test_failing_filter(self):
        conf = {
            "assertion_builder": {
                "type": "pysaml",
                "issuer": "http://myradius.org/",
                "lifetime": 600,
            },
            'attribute_filters': [
                {
                    'type': 'combine_error',
                }
            ]
        }
        with self.assertRaises(AbfabIDPError):
            AbfabIDP(conf)

    def test_failing_source(self):
        conf = {
            "assertion_builder": {
                "type": "pysaml",
                "issuer": "http://myradius.org/",
                "lifetime": 600,
            },
            "attribute_sources": [
                {
                    "type": "ldap",
                    'ldap_uri': "ldap://ldap.forumsys.com:389",
                    'ldap_basedn': "dc=example,dc=com",
                    'ldap_login': 'cn=read-only-admin,dc=example,dc=com',
                    'ldap_password': 'passwordFAKE',
                    'ldap_mapping': "(uid=%u)",
                }
            ]
        }
        idp = AbfabIDP(conf)
        code, attributes, _ = idp.post_auth([
            ('User-Name', 'alex@test.org'),
            ('RADATTR1', 'whatever'),
            ('RADATTR2', 'whatever'),
            ('RADATTR3', 'whatever'),
            ('RADATTR4', 'whatever'),
        ])
        self.assertEqual(code, radiusd.RLM_MODULE_OK)


if __name__ == '__main__':
    unittest.main()
